/**
 * Editor for builder
 * @class ApBuilderEditor
 */

'use strict'

import React, {PropTypes as types} from 'react'
import classnames from 'classnames'
import {ApEditor} from 'apeman-react-editor'
import {ApButtonGroup, ApButton} from 'apeman-react-button'
/** @lends ApBuilderEditor */
const ApBuilderEditor = React.createClass({
  // --------------------
  // Specs
  // --------------------
  propTypes: {
    /** Handle for change */
    onChange: types.func,
    /** Handle for cancel */
    onCancel: types.func,
    /** Handle for submit */
    onSubmit: types.func,
    /** Label texts */
    labels: types.object,
    /** Editting source */
    src: types.string,
    /** Editting type */
    mode: types.string,
    /** Error message */
    error: types.oneOfType([ types.string, types.object ]),
    /** Show spinner */
    spinning: types.bool

  },

  getDefaultProps () {
    return {
      labels: {
        cancel: 'Cancel',
        submit: 'Submit'
      }
    }
  },

  render () {
    const s = this
    let { state, props } = s
    let { labels, src } = props
    return (
      <div className={ classnames('ap-builder-editor', props.className) }>
        <div className='ap-builder-editor-main'>
          <ApEditor value={ src }
                    spinning={ props.spinning }
                    mode={ props.mode }
                    error={ props.error }
                    onChange={ props.onChange }
                    editorRef={ (editor) => s.editor = editor }
          />
        </div>
        <div className='ap-builder-editor-control'>
          <ApButtonGroup>
            <ApButton onTap={ s.handleCancel }> { labels.cancel }</ApButton>
            <ApButton primary onTap={ s.handleSubmit }> { labels.submit }</ApButton>
          </ApButtonGroup>
        </div>
      </div>
    )
  },
  // --------------------
  // Lifecycle
  // --------------------

  componentDidUpdate (prevProps, prevState) {
    const s = this
    let { editor } = s
    if (editor) {
      editor.refresh()
    }
  },

  // --------------------
  // Handle
  // --------------------

  handleCancel (e) {
    const s = this
    let { props } = s
    let { mode, src } = props
    if (props.onCancel) {
      props.onCancel({ mode, value: src })
    }
  },

  handleSubmit (e) {
    const s = this
    let { props } = s
    let { mode, src } = props
    if (props.onSubmit) {
      props.onSubmit({ mode, value: src })
    }
  }
})

export default ApBuilderEditor
